==> OpenCL-0.8.5.tgz: OpenCL-0.8.5/checks/block.cpp <==
/* This file is in the public domain */

/*
  If defined, make sure decryption actually works (ie is the inverse of the
  encryption function). But it makes the benchmarks really slow, because of all
  the overhead for checking the decryption. So it's normally disabled (don't
  worry, it's turned on once in a while to make sure nothing is broken).
*/
//#define CHECK_DECRYPTION

// I added this warning after once spending 4 hours searching around trying to
// find out why the block ciphers were going so slow. It sucked.
#ifdef CHECK_DECRYPTION
#warning "WARNING: Checking decryption. This will slow down the benchmarks!"
#endif

#include <iostream>
#include <string>
#include <cstdlib>

#include <opencl/filter.h>
#include <opencl/symkey.h>
#include <opencl/lookup.h>

#include <opencl/lion.h>
#include <opencl/lubyrack.h>

using namespace OpenCL;

#ifdef CHECK_DECRYPTION
#include <opencl/crc32.h>

typedef OpenCL::CRC32 CHECK_HASH;

class ECB_Encryption_ErrorCheck : public Filter
   {
   public:
      void write(const byte[], u32bit);
      void end_msg();
      ECB_Encryption_ErrorCheck(BlockCipher* c, const BlockCipher_Key& key)
         : BLOCKSIZE(c->BLOCK_SIZE), cipher(c)
         {
         buffer.create(BLOCKSIZE);
         cipher->set_key(key);
         position = 0;
         }
   private:
      const u32bit BLOCKSIZE;
      BlockCipher* cipher;
      SecureVector<byte> buffer;
      u32bit position;
      CHECK_HASH hash_input, hash_decrypt;
   };

void ECB_Encryption_ErrorCheck::write(const byte input[], u32bit length)
   {
   hash_input.update(input, length);
   buffer.copy(position, input, length);
   if(position + length >= BLOCKSIZE)
      {
      cipher->encrypt(buffer);
      send(buffer, BLOCKSIZE);
      cipher->decrypt(buffer);
      hash_decrypt.update(buffer, BLOCKSIZE);
      input += (BLOCKSIZE - position);
      length -= (BLOCKSIZE - position);
      while(length >= BLOCKSIZE)
         {
         cipher->encrypt(input, buffer);
         send(buffer, BLOCKSIZE);
         cipher->decrypt(buffer);
         hash_decrypt.update(buffer, BLOCKSIZE);
         input += BLOCKSIZE;
         length -= BLOCKSIZE;
         }
      buffer.copy(input, length);
      position = 0;
      }
   position += length;
   }

void ECB_Encryption_ErrorCheck::end_msg()
   {
   /* We want to check decryption works too, so we just have a hash of the
      decrypted output. If it doesn't equal the hash of the input, we're in
      trouble. It's done internally, so we don't have to export this odd
      inconsistenctey WRT the other functions into the validate code. If it's
      wrong, we just go and die. */

   SecureVector<byte> input_hash = hash_input.final();
   SecureVector<byte> output_hash = hash_decrypt.final();

   for(u32bit j = 0; j != input_hash.size(); j++)
      if(input_hash[j] != output_hash[j])
         {
         std::cout << "In " << cipher->name()
                 << " decryption CRC check failed." << std::endl;
         break;
         }

   if(position)
      throw Exception("ECB: input was not in full blocks");
   }
#else
#include <opencl/ecb.h>
#endif

Filter* lookup_block(const std::string& algname, const BlockCipher_Key& key)
   {
#ifdef CHECK_DECRYPTION
   #define ECB_Filter ECB_Encryption_ErrorCheck
#else
   #define ECB_Filter ECB_Encryption
#endif

   Filter* cipher = 0;
   try {
     cipher = new ECB_Filter(algname, key);
   }
   catch(Algorithm_Not_Found) {}

   if(cipher)
      return cipher;

   /* These are kind of special purpose (for benchmarking). Thus the large
       block size.
   */
   if(algname == "Lion(SHA-1,SEAL)")
      return new ECB_Filter(new Lion("SHA-1", "SEAL", 256*1024), key);
   else if(algname == "Lion(MD5,ISAAC)")
      return new ECB_Filter(new Lion("MD5", "ISAAC", 256*1024), key);

   else if(algname == "Lion(SHA-1,SEAL,64)")
      return new ECB_Filter(new Lion("SHA-1", "SEAL", 64), key);
   else if(algname == "Luby-Rackoff(SHA-1)")
      return new ECB_Filter(new LubyRackoff("SHA-1"), key);

   else return 0;
   }

==> OpenCL-0.8.6.tgz: OpenCL-0.8.6/checks/block.cpp <==
/* This file is in the public domain */

/*
  If defined, make sure decryption actually works (ie is the inverse of the
  encryption function). But it makes the benchmarks really slow, because of all
  the overhead for checking the decryption. So it's normally disabled (don't
  worry, it's turned on once in a while to make sure nothing is broken).
*/
//#define CHECK_DECRYPTION

// I added this warning after once spending 4 hours searching around trying to
// find out why the block ciphers were going so slow. It sucked.
#ifdef CHECK_DECRYPTION
#warning "WARNING: Checking decryption. This will slow down the benchmarks!"
#endif

#include <iostream>
#include <string>
#include <cstdlib>

#include <opencl/filter.h>

using namespace OpenCL;

#ifdef CHECK_DECRYPTION
#include <opencl/lookup.h>

class ECB_Encryption_ErrorCheck : public Filter
   {
   public:
      void write(const byte[], u32bit);
      void end_msg();
      ECB_Encryption_ErrorCheck(const std::string& cipher_name,
                                const BlockCipher_Key& key) :
         BLOCKSIZE(block_size_of(cipher_name))
         {
         cipher = get_block_cipher(cipher_name);
         input_hash = get_hash("SHA-1");
         decrypt_hash = get_hash("SHA-1");
         buffer.create(BLOCKSIZE);
         cipher->set_key(key);
         position = 0;
         }
      ~ECB_Encryption_ErrorCheck()
         { delete cipher; delete input_hash; delete decrypt_hash; }
   private:
      const u32bit BLOCKSIZE;
      BlockCipher* cipher;
      SecureVector<byte> buffer;
      u32bit position;
      HashFunction* input_hash, *decrypt_hash;
   };

void ECB_Encryption_ErrorCheck::write(const byte input[], u32bit length)
   {
   input_hash->update(input, length);
   buffer.copy(position, input, length);
   if(position + length >= BLOCKSIZE)
      {
      cipher->encrypt(buffer);
      send(buffer, BLOCKSIZE);
      cipher->decrypt(buffer);
      decrypt_hash->update(buffer, BLOCKSIZE);
      input += (BLOCKSIZE - position);
      length -= (BLOCKSIZE - position);
      while(length >= BLOCKSIZE)
         {
         cipher->encrypt(input, buffer);
         send(buffer, BLOCKSIZE);
         cipher->decrypt(buffer);
         decrypt_hash->update(buffer, BLOCKSIZE);
         input += BLOCKSIZE;
         length -= BLOCKSIZE;
         }
      buffer.copy(input, length);
      position = 0;
      }
   position += length;
   }

void ECB_Encryption_ErrorCheck::end_msg()
   {
   SecureVector<byte> hash1 = input_hash->final();
   SecureVector<byte> hash2 = decrypt_hash->final();

   if(hash1 != hash2)
      {
      std::cout << "In " << cipher->name()
                << " decryption check failed." << std::endl;
      }

   if(position)
      throw Exception("ECB: input was not in full blocks");
   }
#else
#include <opencl/ecb.h>
#endif

Filter* lookup_block(const std::string& algname, const std::string& key)
   {
#ifdef CHECK_DECRYPTION
   #define ECB_Filter ECB_Encryption_ErrorCheck
#else
   #define ECB_Filter ECB_Encryption
#endif

   Filter* cipher = 0;
   try {
     cipher = new ECB_Filter(algname, key);
   }
   catch(Algorithm_Not_Found) {}

   return cipher;
   }
